#!BPY

"""
Name: 'DEPTH OF FIELD'
Blender: 234
Group: 'Wizards'
Tooltip: 'Create Depth of Field effect'
"""

######################################################
# DOF.py v0.1
#
# Script to create depth of field effect
#
#####################################################
# This script create a depth of field effect on an 
# image 
# How to use it :
# - Create an empty named 'Focus'
# - Place it where you want the focus to be
# - Select the camera you want to use 
# - Launch the script (Alt-P)
# - Set up variables : 
#   "amount"  must be between 0.2 and 2 
#   "quality"  must belong to [0..3]
# - Press "Render"
######################################################
# Author : Jean-Baptiste PERIN (jb_perin@yahoo.fr)
######################################################

import math
from math import *
import Blender
from Blender import *

tabosa = [5,8,11,16]


print "****START****"

scn = Scene.getCurrent()
curframe = Get("curframe")

#########
# Cette fonction renvoie l'objet de nom name
# in  : 
# out : l'objet de nom name ou None
#########
def getObject(name):
	p = None
	objs = Blender.Object.Get()
	for o in objs:
		if o.getName()==name:
			p = o
			break;
	return p

#########
# Cette fonction renvoie l'objet focus
# in  : 
# out : l'objet focus
#########
def getFocus():
	target=Object.Get('Focus')
	return (target)

#########
# Cette fonction renvoie la position du focus
# in  : 
# out : coordonees X,Y,Z du focus
#########
def getFocusPos():
	target=getFocus()
	tarpos= target.getMatrix('worldspace').translationPart()
	return (tarpos)

#########
# Cette fonction renvoie la camera selectionnee
# in  : 
# out : l'objet camera selectionnee
#########
def getCamera():
	objs=Object.GetSelected()
	return objs[0]

#########
# Cette fonction renvoie la position de la camera
# in  : 
# out : coordonees X,Y,Z de la camera
#########
def getCameraPos():
	cam=getCamera()
	campos= cam.getMatrix('worldspace').translationPart()
	return campos

#########
# Cette fonction calcule l'orientation entre 2 positions
# in  : tarpos : coordonnees X,Y,Z de la cible
#	    oripos : coordonnees X,Y,Z de l'origine
# out : coordonees X,Y,Z du vecteur orientation (vecteur normalise)
#########
def computeOrientation(oripos, tarpos):
	vec = Mathutils.Vector([tarpos[0]-oripos[0], tarpos[1]-oripos[1],tarpos[2]-oripos[2]])
	vec.normalize()
	return vec

#########
# Cette fonction calcule la matrice de transformation a appliquer pour que
#   le vecteur Z d'un repere en 'oripos' pointe vers un point situe en 'tarpos'
# in  : tarpos : coordonnees X,Y,Z de la cible
#	    oripos : coordonnees X,Y,Z de l'origine
# out : matrice de transformation orientation (vecteur normalise)
#########
def computeTransformationMatrix(oripos, tarpos):

	vec = computeOrientation(oripos, tarpos)
	angleY = asin(-vec[0])
	angleX = acos(vec[2]/cos(angleY))

	rxmat = Blender.Mathutils.RotationMatrix(degrees(angleX), 4, 'x')
	rymat = Blender.Mathutils.RotationMatrix(degrees(angleY), 4, 'y')

	mat=rxmat*rymat
	mat.transpose()
	return (mat)


#########
# Cette procedure cree une camera et l'ajoute a la scene
# in  : 
# out : la camera cree
#########
def createCamera():
	global scn
	currentcam = getCamera().getData()
	obcam=Object.New('Camera')
	ncamera=Camera.New('persp','LaCamera')
	ncamera.setClipEnd(currentcam.getClipEnd())
	ncamera.setClipStart(currentcam.getClipStart())
	ncamera.setLens(currentcam.getLens())
	if (currentcam.getType() == 0):
		ncamera.setType('persp')
	else:
		ncamera.setType('ortho')
	obcam.link(ncamera)
	scn.link(obcam)
	scn.setCurrentCamera(obcam)
	return (obcam)

#########
# Cette procedure configure le rendu de la scene
# in  : 
# out : 
#########
def configureRenderingContext():
	global scn, nb
   
	context=scn.getRenderingContext()

	context.enableOversampling(1)
	context.enableMotionBlur(1)
	context.motionBlurLevel(0.5)
	context.enableUnifiedRenderer(1)
	context.setOversamplingLevel(nb)


#########
# Cette procedure restaure les parametres de rendu initiaux
# in  : 
# out : 
#########
def restoreRenderingContext():
	global scn, nb
   
	context=scn.getRenderingContext()

	context.enableOversampling(0)
	context.enableMotionBlur(0)
	context.motionBlurLevel(0.5)
	context.enableUnifiedRenderer(0)
	context.setOversamplingLevel(8)


def Main():
	global scn, curframe, nb, diametre

	target = getFocus()
	tarpos = getFocusPos()
	campos = getCameraPos()
	mat = computeTransformationMatrix(campos, tarpos)
	##-----------
	## Creation de la camera pour le rendu
	##-----------
	obcam=createCamera()

	##-----------
	## Creation des ipos pour la camera de rendu
	##-----------
	myipo = Blender.Ipo.New('Object','ipo'+obcam.getName())
	obcam.setIpo(myipo)
	camposX = myipo.addCurve('LocX')
	camposY = myipo.addCurve('LocY')
	camposZ = myipo.addCurve('LocZ')
	obcam.makeTrack(target, 0)

	##-----------
	## Animation de la camera de rendu
	##-----------
	for ii in range(nb+1):
		angle = ii*((2*pi)/nb)
		x=diametre*cos(angle)
		y=diametre*sin(angle)
		camrelpos = Mathutils.Vector([x, y, 0.0, 0.0])
		camabspos = Mathutils.VecMultMat(camrelpos, mat)
		camposX.addBezier(((curframe+ii/nb), campos[0]+camabspos[0]))
		camposY.addBezier(((curframe+ii/nb), campos[1]+camabspos[1]))
		camposZ.addBezier(((curframe+ii/nb), campos[2]+camabspos[2]))

	camposX.Recalc()
	camposY.Recalc()
	camposZ.Recalc()
	camposX.setInterpolation('Linear')
	camposY.setInterpolation('Linear')
	camposZ.setInterpolation('Linear')

	configureRenderingContext()
	context=scn.getRenderingContext()
	context.render()
	restoreRenderingContext()
	cam=getCamera()
	scn.setCurrentCamera(cam)
	Redraw()


print "****END****"

########################################################################
#
#            GUI FUNCTIONS AND VARIABLES
#
########################################################################

Amount = Blender.Draw.Create(0)
Quality = Blender.Draw.Create(0)
Msg = ' '

def event (evt, val):
	if evt == Blender.Draw.ESCKEY:
		Blender.Draw.Exit()
	return

def button_event(evt):
	global Amount, Quality, Msg
	global nb, diametre
	if evt==1:
		amount = eval(Amount.val)
		osa =  eval(Quality.val)
		if 0>=amount:
			Msg = 'Amount must be greater than 0.0 (around 1.5 recommended)'
		else:
			obs = Object.GetSelected()
			if (len(obs)!=1): 
				Msg = 'One and only one camera can be selected ...'
			else:
				if obs[0].getType()!='Camera':
					Msg = 'The selected object must be a camera'
				else: 
					if (0>osa or osa>3):
						Msg = 'Quality must belong to [0..3]'
					else:
						leFocus = getObject('Focus')
						if leFocus==None:
							Msg = 'there must exist an object named Focus in the scene'	
						else:
							nb=tabosa[osa]
							diametre = amount 
							Blender.Draw.Exit()
							Main()
				
		Blender.Draw.Redraw(1)

def GUI():
	global Amount, Quality, Msg
	Blender.BGL.glClearColor(0,0,1,1)
	Blender.BGL.glClear(Blender.BGL.GL_COLOR_BUFFER_BIT)
	Blender.BGL.glColor3f(1,1,1)
	Blender.BGL.glRasterPos2i(20,260)
	Blender.Draw.Text ("DEPTH OF FIELD v0.1 by Jean-Baptiste PERIN", 'normal')
	Blender.BGL.glRasterPos2i(20,240)
	Blender.Draw.Text ("- Create an empty named 'Focus'", 'normal')
	Blender.BGL.glRasterPos2i(20,220)
	Blender.Draw.Text ("- Place it where you want the focus to be", 'normal')
	Blender.BGL.glRasterPos2i(20,200)
	Blender.Draw.Text ("- Select the camera you want to use", 'normal')
	Blender.BGL.glRasterPos2i(20,180)
	Blender.Draw.Text ("- Press button 'Render'", 'normal')
	Amount = Blender.Draw.String("Amount: ", -1, 20, 100, 250, 20, '1.0', 100)
	Quality = Blender.Draw.String("Quality [0..3]: ", -1, 20, 75, 250, 20, '1', 100)
	Blender.Draw.Toggle("Render", 1, 20, 10, 100, 20, 0, "Render Image")
	Blender.BGL.glRasterPos2i(20,40)
	Blender.Draw.Text (Msg, 'normal')


Blender.Draw.Register(GUI, event, button_event)